
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2022 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_DEVICEINFO_H
#define ITALA_API_DEVICEINFO_H

#include "ItalaApi/InterfaceInfo.h"
#include "ItalaApi/ItalaDefs.h"

namespace Itala {

struct DeviceInfoData;

/**
 * This class stores information about a device and represents its
 * identity. When an enumeration (also known as "discovery") is performed via
 * the ISystem interface, a list of DeviceInfo is returned as result. To
 * initialize and use a particular connected device, the user must pass the
 * corresponding DeviceInfo instance to the ISystem interface.
 */
class ITALA_API DeviceInfo {
public:
  /**
   * Get the unique ID of the device. The ID of a discovered device is a
   * GenICam::gcstring assigned and used by the internal transport layer to
   * uniquely identify the device in the current execution context. The ID
   * string isn't regulated by any standard and its format and content are
   * implementation specific.
   *
   * @return Unique ID of the device
   */
  GenICam::gcstring ID() const;

  /**
   * Get a string containing the name of the device vendor.
   *
   * @return Name of the device vendor
   */
  GenICam::gcstring Vendor() const;

  /**
   * Get a string containing the model of the device.
   *
   * @return Model of the device
   */
  GenICam::gcstring Model() const;

  /**
   * Get a string containing the user readable name of the device.
   *
   * @return Name of the device
   */
  GenICam::gcstring DisplayName() const;

  /**
   * Get a string containing the name of the device defined by the user, when
   * available.
   *
   * @return User-defined name of the device
   */
  GenICam::gcstring UserDefinedName() const;

  /**
   * Get a string containing the serial number of the device.
   *
   * @return Serial number of the device
   */
  GenICam::gcstring SerialNumber() const;

  /**
   * Get a string containing the version of the physical device.
   *
   * @return Version of the device
   */
  GenICam::gcstring Version() const;

  /**
   * Get the accessibility status of the device. Depending on its network
   * configuration or internal state (e.g. if already in use by another
   * process) its accessibility may change. It's responsibility of the user to
   * check if a device is currently available for initialization before using
   * it.
   *
   * @return access status of the device.
   */
  DeviceAccessStatus AccessStatus() const;

  /**
   * Get an unsigned 32 bit integer containing the IP address of the device in
   * host order. For instance, the value 0xC0A8012A represents the IP address
   * 192.168.1.42 in IPv4 dotted-decimal notation.
   *  - 0xC0 = 192
   *  - 0xA8 = 168
   *  - 0x01 = 1
   *  - 0x2A = 42
   *
   * @return IP address of the device as integer in host byte order.
   */
  uint32_t IpAddress() const;

  /**
   * Get the subnet mask configured on the device in the same format described
   * in IpAddress().
   *
   * @return Subnet mask of the device as raw integer in host byte order.
   *
   * @see IpAddress()
   */
  uint32_t SubnetMask() const;

  /**
   * Get the default gateway configured on the device according with the same
   * format described in IpAddress().
   *
   * @return Default gateway of the device as raw integer in host byte order.
   *
   * @see IpAddress()
   */
  uint32_t DefaultGateway() const;

  /**
   * Get an unsigned 64 bit integer containing the mac address of the device in
   * host byte order. A 64 bit integer is required since the MAC address is 6 bytes
   * long. For instance, the value 0x0000885FE8200001 represents the mac address
   * 88:5f:e8:20:00:01 in colon-heaxadecimal notation.
   *
   * @return MAC address of the device as integer in host byte order.
   */
  uint64_t MacAddress() const;

  /**
   * Get the InterfaceInfo instance of the interface under which the device is
   * connected and enumerated.
   *
   * @return Info object of the interface which owns the device.
   */
  InterfaceInfo ParentInterfaceInfo() const;

private:
  DeviceInfoData* op;

public:
  DeviceInfo();
  DeviceInfo(DeviceInfoData* data);
  DeviceInfo(const DeviceInfo& other);
  DeviceInfo(DeviceInfo&& other);
  DeviceInfo& operator=(const DeviceInfo& other);
  DeviceInfo& operator=(DeviceInfo&& other);
  ~DeviceInfo();
  bool operator==(const DeviceInfo& other);
  DeviceInfoData& GetOp();
};

typedef std::vector<DeviceInfo> DeviceInfoList;
}  // namespace Itala

#endif