
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2022 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_IIMAGE_H
#define ITALA_API_IIMAGE_H

#include <cstdint>
#include "ItalaApi/Export.h"
#include "GenApi/INode.h"
#include "ItalaApi/IDisposable.h"
#include "PFNC/PFNC.h"

namespace Itala {

/**
 * Image interface. It represents the image buffer grabbed from the device. The ownership of the
 * image instance belongs to the client code, so it needs to be disposed when it's no longer
 * required via IImage::Dispose.
 */
struct IImage : public IDisposable {
public:
  /**
   * Get the width of the image in pixels.
   */
  virtual size_t GetWidth() const = 0;

  /**
   * Get the height of the image in pixels.
   */
  virtual size_t GetHeight() const = 0;

  /**
   * Get the offsex along the X axis of the image, in pixels.
   */
  virtual size_t GetOffsetX() const = 0;

  /**
   * Get the offsex along the Y axis of the image, in pixels.
   */
  virtual size_t GetOffsetY() const = 0;

  /**
   * Get the padding along the X axis of the image, in pixels.
   */
  virtual size_t GetPaddingX() const = 0;

  /**
   * Get the padding along the Y axis of the image, in pixels.
   */
  virtual size_t GetPaddingY() const = 0;

  /**
   * Get the pixel format of the image. The PFNC standard from GenICam is used to identify the
   * format.
   */
  virtual PfncFormat GetPixelFormat() const = 0;

  /**
   * Get the ID of the image.
   */
  virtual uint64_t GetFrameID() const = 0;

  /**
   * Get the timestamp of the image.
   */
  virtual uint64_t GetTimestamp() const = 0;

  /**
   * Get the payload size of the image, in bytes.
   */
  virtual size_t GetPayloadSize() const = 0;

  /**
   * Reports wether the grabbed image is incomplete, i.e. some bytes are missing due to transfer
   * issues.
   */
  virtual bool IsIncomplete() const = 0;

  /**
   * Get the amount of bytes actually stored in the buffer.
   */
  virtual size_t GetBytesFilled() const = 0;

  /**
   * Get the pointer to the memory buffer containing the grabbed data. The content is retrieved
   * directly from the device and, if chunk data functionality is enabled, the chunks are also
   * included in the buffer.
   */
  virtual void* GetData() const = 0;

  /**
   * Report wether the buffer contains chunk data or not.
   */
  virtual bool HasChunkData() const = 0;

  /**
   * Get the specified node from the internal chunk data nodemap.
   */
  virtual GenApi::INode* GetChunkNode(GenICam::gcstring name) const = 0;

  /**
   * Creates a new image cloned from the current one with its own allocated memory. It needs to be
   * disposed when it's no longer required via Dispose().
   *
   * @warning
   * Chunk data access via GenApi is not available for cloned images: if the source image is
   * directly coming from a grab, it's highly recommended to manually retrieve the needed chunk
   * values before the source image gets disposed.
   *
   * @return Pointer to the cloned image instance.
   */
  virtual IImage* Clone() const = 0;

  /**
   * Creates a new image converted from the current one with its own allocated memory and the
   * specified pixel format. It needs to be disposed when it's no longer required via Dispose().
   *
   * @warning
   * Chunk data access via GenApi is not available for converted images: if the source image is
   * directly coming from a grab, it's highly recommended to manually retrieve the needed chunk
   * values before the source image gets disposed.
   *
   * @warning
   * When converting from packed to unpacked formats, the resulting data is aligned according to the
   * PFNC specification. Most of the time the resulting data is LSB aligned, e.g. a 12-bit pixel
   * value stored in a 16-bit component has its MSBs padded with zeroes. See the official PFNC
   * document hosted by EMVA for details.
   *
   * @return Pointer to the converted image instance.
   */
  virtual IImage* Convert(PfncFormat dest_format) const = 0;

protected:
  virtual ~IImage() {}
};

/**
 * Creates a new image from a memory buffer by copying and taking ownership of the data. The size in
 * bytes of the buffer is obtained from the width, height and format provided by the caller. For
 * that reason, Y padding in the buffer is ignored while X padding is not supported.
 *
 * @note
 * Chunk data access and grab related info are not available for created images.
 *
 * @param data Pointer to the image data.
 * @param width Width of the image in pixel.
 * @param height Height of the image in pixel.
 * @param pixelFormat Pixel format of the image in pixel.
 *
 * @return Pointer to the converted image instance.
 */
ITALA_API IImage* CreateImage(void* data, size_t width, size_t height, PfncFormat pixelFormat);

}  // namespace Itala

#endif