
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2022 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_INTERFACEINFO_H
#define ITALA_API_INTERFACEINFO_H

#include <vector>
#include "Base/GCString.h"
#include "ItalaApi/Export.h"

namespace Itala {

struct InterfaceInfoData;

/**
 * This class stores information about an available interface and represents
 * its identity. When an enumeration is performed via the ISystem interface, a
 * list of InterfaceInfo is returned as result. To enumerate the available
 * devices under a specific interface, the user must pass the corresponding
 * InterfaceInfo instance to the dedicated ISystem::EnumerateDevices function.
 */
class ITALA_API InterfaceInfo {
public:
  /**
   * Get the unique ID of the interface. The ID of an interface is a
   * GenICam::gcstring assigned and used by the internal transport layer to
   * uniquely identify the interface in the current execution context. The ID
   * string isn't regulated by any standard and its format and content are
   * implementation specific.
   *
   * @return Unique ID of the interface
   */
  GenICam::gcstring ID() const;

  /**
   * Get a string containing the user readable name of the interface.
   *
   * @return Name of the interface
   */
  GenICam::gcstring DisplayName() const;

  /**
   * Get an unsigned 32 bit integer containing the IP address of the interface
   * in host byte order. For instance, the value 0xC0A8012A represents the IP
   * address 192.168.1.42 in IPv4 dotted-decimal notation.
   *  - 0xC0 = 192
   *  - 0xA8 = 168
   *  - 0x01 = 1
   *  - 0x2A = 42
   *
   * @return IP address of the interface as integer in host byte order.
   */
  uint32_t IpAddress() const;

  /**
   * Get an unsigned 64 bit integer containing the mac address of the
   * interface in host byte order. A 64 bit integer is required since the MAC
   * address is 6 bytes long. For instance, the value 0x0000885FE8200001
   * represents the mac address 88:5f:e8:20:00:01 in colon-heaxadecimal
   * notation.
   *
   * @return MAC address of the integer as integer in host byte order.
   */
  uint64_t MacAddress() const;

  /**
   * Get the subnet mask of the interface in the same format described in
   * IpAddress().
   *
   * @return Subnet mask of the device as integer in host byte order.
   *
   * @see IpAddress()
   */
  uint32_t SubnetMask() const;

public:
  InterfaceInfo();
  InterfaceInfo(InterfaceInfoData* data);
  InterfaceInfo(const InterfaceInfo& other);
  InterfaceInfo(InterfaceInfo&& other);
  InterfaceInfo& operator=(const InterfaceInfo& other);
  InterfaceInfo& operator=(InterfaceInfo&& other);
  ~InterfaceInfo();
  bool operator==(const InterfaceInfo& other);
  InterfaceInfoData& GetOp();

private:
  InterfaceInfoData* op;
};

typedef std::vector<InterfaceInfo> InterfaceInfoList;
}  // namespace Itala

#endif