
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2024 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_POLARIZATION_H
#define ITALA_API_POLARIZATION_H

#include "ItalaApi/Export.h"
#include "PFNC/PFNC.h"

namespace Itala {
struct IImage;

/**
 * Components of the polarizer filter which represent a specific angle of polarization of the
 * incoming light.
 */
enum class PolarComponent { P0, P45, P90, P135 };

/**
 * Demosaicing algorithm applied to the image. None, NearestNeighbour and Bilinear
 * algorithms give a resulting image with the same pixel format of the original. WeightedBilinear
 * returns 32-bit floating point data. The width and height of the resulting image also depend on
 * the used algorithm. In particular:
 * - None -> w/2, h/2
 * - NearestNeighbour -> w - 2, h - 2
 * - Bilinear -> w - 3, h - 3
 * - WeightedBilinear -> w - 4, h - 4
 */
enum class PolarDemosaicingAlgorithm { None, NearestNeighbour, Bilinear, WeightedBilinear };

/**
 * Extract a specific component (or angle) of the polarizer filter. The resulting image
 * represents the captured light at the angle specified in degrees.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw image, acquired using a monochrome polarization filter array.
 * @param component Component of the polarizer sensor.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Pointer to the image containing the requested polar component.
 */
ITALA_API IImage* ExtractPolarComponent(const IImage* image, PolarComponent component,
                                        PolarDemosaicingAlgorithm algorithm);
/**
 * Convenient POD structure used as container for @ref ExtractAllPolarComponents 's
 * multi-result.
 */
struct PolarComponents {
  IImage *P0, *P45, *P90, *P135;
};
/**
 * Extract an image for each component (or angle) of the polarizer filter. Each resulting
 * image represents the captured light at 0, 45, 90 and 135 degrees respectively.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw image, acquired using a monochrome polarization filter array.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Structure containing the pointers to the polar component images.
 */
ITALA_API PolarComponents ExtractAllPolarComponents(const IImage* image,
                                                    PolarDemosaicingAlgorithm algorithm);
/**
 * Convenient POD structure used as container for @ref ExtractAllPolarComponentsColor 's
 * multi-result.
 */
struct PolarComponentsColor {
  PolarComponents RedComponents, GreenComponents, BlueComponents;
};
/**
 * For each color channel, extract an image for each component (or angle) of the polarizer
 * filter. Each resulting image represents the captured light at 0, 45, 90 and
 * 135 degrees respectively, for each color channel.
 *
 * @note Only bayer color images are supported.
 *
 * @param image Raw image, acquired using a color polarization filter array.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Structure containing the pointers to the polar component images, for each color channel.
 */
ITALA_API PolarComponentsColor ExtractAllPolarComponentsColor(const IImage* image,
                                                              PolarDemosaicingAlgorithm algorithm);
/**
 * Computes only the S0 component of the Stokes vectors from the extracted polar components.
 *
 * @param polarComponents Polarization components demosaiced from the raw image.
 *
 * @return Pointer to the image containing the requested stokes vectors.
 */
ITALA_API IImage* ComputeStokesS0(const PolarComponents& polarComponents);

/**
 * Computes only the S1 component of the Stokes vectors from the extracted polar components.
 *
 * @param polarComponents Polarization components demosaiced from the raw image.
 *
 * @return Pointer to the image containing the requested stokes vectors.
 */
ITALA_API IImage* ComputeStokesS1(const PolarComponents& polarComponents);

/**
 * Computes only the S2 component of the Stokes vectors from the extracted polar components.
 *
 * @param polarComponents Polarization components demosaiced from the raw image.
 *
 * @return Pointer to the image containing the requested stokes vectors.
 */
ITALA_API IImage* ComputeStokesS2(const PolarComponents& polarComponents);

/**
 * Convenient POD structure used as container for @ref ComputeAllStokes 's
 * multi-result.
 */
struct StokesVectors {
  IImage *S0, *S1, *S2;
};
/**
 * Compute all the Stokes vectors to determine the polarization state of the light given the
 * extracted components. The Stokes vector components are expressed as 32-bit or 64-bit floating
 * point numbers depending on the raw image format.
 *
 * @param polarComponents Polarization components demosaiced from the raw image.
 *
 * @return Structure containing the pointers to the stokes vector images.
 */
ITALA_API StokesVectors ComputeAllStokes(const PolarComponents& polarComponents);

/**
 * Compute the AoLP (Angle of Linear Polarization) image from the Stokes vectors.
 *
 * @param stokesVectors Stokes vectors computed from the polarization components.
 *
 * @return Pointer to the AoLP image.
 */
ITALA_API IImage* ComputeAoLP(const StokesVectors& stokesVectors);

/**
 * Compute the DoLP (Degree of Linear Polarization) image from the Stokes vectors.
 *
 * @param stokesVectors Stokes vectors computed from the polarization components.
 *
 * @return Pointer to the DoLP image.
 */
ITALA_API IImage* ComputeDoLP(const StokesVectors& stokesVectors);

/**
 * Compute the intensity image from the Stokes vectors.
 *
 * @param stokesVectors Stokes vectors computed from the polarization components.
 * @param rawImagePixelFormat The format of the raw polarization image.
 *
 * @return Pointer to intensity image.
 */
ITALA_API IImage* ComputeIntensity(const StokesVectors& stokesVectors,
                                   PfncFormat rawImagePixelFormat);
/**
 * Convenience function which computes the AoLP (Angle of Linear Polarization) image directly
 * from the raw polarization image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw polarization image.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Pointer to AoLP image.
 */
ITALA_API IImage* ComputeAoLP(const IImage* image, PolarDemosaicingAlgorithm algorithm);

/**
 * Convenience function which computes the DoLP (Degree of Linear Polarization) image
 * directly from the raw polarization image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw polarization image.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Pointer to DoLP image.
 */
ITALA_API IImage* ComputeDoLP(const IImage* image, PolarDemosaicingAlgorithm algorithm);

/**
 * Convenience function which computes the intensity image directly from the raw polarization
 * image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw polarization image.
 * @param algorithm Demosaicing algorithm to be applied to the image.
 *
 * @return Pointer to intensity image.
 */
ITALA_API IImage* ComputeIntensity(const IImage* image, PolarDemosaicingAlgorithm algorithm);

/**
 * Computes an image split in quadrants containing the four extracted polarization
 * components (angles), of the same width and height of the raw polarization image.
 *
 * @note Only monochrome images are supported.
 *
 * @param image Raw polarization image.
 *
 * @return Pointer to quadrants image.
 */
ITALA_API IImage* ComputePolarQuadrantsImage(const IImage* image);

}  // namespace Itala

#endif