
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2022 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_SMART_H
#define ITALA_API_SMART_H

#include <memory>
#include "ItalaApi/IDevice.h"
#include "ItalaApi/IImage.h"
#include "ItalaApi/ISystem.h"
#include "ItalaApi/Polarization.h"

/**
@file
* The following functions provide a convenient way for managing IDisposable
* instances lifecycle via the smart pointers of the C++ standard library.
* These functions should be called immediately after creating the instance to avoid
* both raw and smart pointers around at the same time.
* When the smart pointer is destroyed, the IDisposable::Dispose method of the IDisposable
* interface is automatically called.
*
* \code{.cpp}
* {
*   // here the system instance is created and passed to a smart pointer
* 	Itala::SystemPtr pSystem = Itala::SmartAssign(Itala::CreateSystem());
* } // here the system instance is automatically disposed as the pointer goes out of scope
* \endcode
*
* @warning When a IDisposable instance is owned by a smart pointer, its
* IDisposable::Dispose method must NEVER be called manually!
*
* Note that this header is just a quick helper completely editable by the user.
* Feel free to introduce improvements or entirely swap the smart pointer
* implementation with a different one (e.g. Boost).
*/

namespace Itala {

typedef std::shared_ptr<ISystem> SystemPtr;
typedef std::shared_ptr<IDevice> DevicePtr;
typedef std::shared_ptr<IImage> ImagePtr;

inline SystemPtr SmartAssign(ISystem* pSystem) {
  return SystemPtr(pSystem, [pSystem](Itala::ISystem* sys) { sys->Dispose(); });
}

inline DevicePtr SmartAssign(IDevice* pDevice) {
  return DevicePtr(pDevice, [pDevice](Itala::IDevice* dev) { dev->Dispose(); });
}

inline ImagePtr SmartAssign(IImage* pImage) {
  return ImagePtr(pImage, [pImage](Itala::IImage* img) { img->Dispose(); });
}

struct SmartPolarComponents {
  ImagePtr P0;
  ImagePtr P45;
  ImagePtr P90;
  ImagePtr P135;
  operator PolarComponents() const { return {P0.get(), P45.get(), P90.get(), P135.get()}; }
};

struct SmartPolarComponentsColor {
  SmartPolarComponents RedComponents, GreenComponents, BlueComponents;
  operator PolarComponentsColor() const { return {RedComponents, GreenComponents, BlueComponents}; }
};

struct SmartStokesVectors {
  ImagePtr S0, S1, S2;
  operator StokesVectors() const { return {S0.get(), S1.get(), S2.get()}; }
};

inline SmartPolarComponents SmartAssign(PolarComponents polarComponents) {
  return {SmartAssign(polarComponents.P0), SmartAssign(polarComponents.P45),
          SmartAssign(polarComponents.P90), SmartAssign(polarComponents.P135)};
}

inline SmartPolarComponentsColor SmartAssign(PolarComponentsColor& polarComponentsColor) {
  return {SmartAssign(polarComponentsColor.RedComponents),
          SmartAssign(polarComponentsColor.GreenComponents),
          SmartAssign(polarComponentsColor.BlueComponents)};
}

inline SmartStokesVectors SmartAssign(StokesVectors stokesVector) {
  return {SmartAssign(stokesVector.S0), SmartAssign(stokesVector.S1), SmartAssign(stokesVector.S2)};
}

}  // namespace Itala

#endif