
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_DEFECTDETECTION_H
#define ITALA_API_C_DEFECTDETECTION_H

#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

/**
 * Dark images are used for both leaky and hotc/cold pixel detection. These images
 * are accumulated via this function. The statistical properties needed
 * for the detection are calculated instantaneously and kept when an image buffer
 * is accumulated. The image buffer memory must be managed by the client code.
 * The image buffer data is passed as generic byte pointer to keep the
 * procedure generic and decoupled from the PFNC formats.
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 * @param[out] pImageBuffer The image buffer data in a generic byte pointer.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_AccumulateDark(H_DEFECTDETECTION hDefDet, uint8_t* pImageBuffer);
/**
 * Gray images are used for hot/cold pixels detection. These images are
 * accumulated via this function. The statistical properties needed
 * for the detection are calculated instantaneously and kept when an image buffer
 * is accumulated. The image buffer memory must be managed by the client code.
 * The image buffer data is passed as generic byte pointer to keep the
 * procedure generic and decoupled from the PFNC formats.
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 * @param[out] pImageBuffer The image buffer data in a generic byte pointer.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_AccumulateGray(H_DEFECTDETECTION hDefDet, uint8_t* pImageBuffer);
/**
 * Flush the results computed for the accumulated dark images. This way a
 * new accumulation and detection can be performed.
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_ResetDark(H_DEFECTDETECTION hDefDet);
/**
 * Flush the results computed for the accumulated gray images. This way a
 * new accumulation and detection can be performed.
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_ResetGray(H_DEFECTDETECTION hDefDet);
/**
 * Run the leaky pixel detection given the accumulated dark images.
 * To retrive how many pxel have been found and retrive them
 * @see DD_GetLeakyCount and @see DD_GetLeakyByIndex .
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 * @param[in] maxFound Maximum amount of leaky pixels to look for.
 * @param[in] threshold Distance from the mean, measured in units of standard deviation, beyond
 * which a pixel is considered defective.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_FindLeaky(H_DEFECTDETECTION hDefDet, uint64_t maxFound, float threshold);
/**
 * Get the amount of leadky detected pixels.
 *
 * @param[out] leakyCount Amount of pixel found.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_GetLeakyCount(size_t* leakyCount);
/**
 * Get a defective pixel given the index.
 *
 * @param[in] index Index of defective pixel to retrive.
 * @param[out] defectivePixel Defective pixel retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_GetLeakyByIndex(size_t index, DefectivePixel* defectivePixel);
/**
 * Run the hot/cold pixel detection given the accumulated gray images.
 * To retrive how many pxel have been found and retrive them
 * @see DD_GetHotNColdCount and @see DD_GetHotNColdByIndex .
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 * @param[in] maxFound Maximum amount of leaky pixels to look for.
 * @param[in] threshold Distance from the mean, measured in units of standard deviation, beyond
 * which a pixel is considered defective.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_FindHotNCold(H_DEFECTDETECTION hDefDet, uint64_t maxFound,
                                       float threshold);
/**
 * Get the amount of hot and cold detected pixels.
 *
 * @param[out] hotNColdCount Amount of pixel found.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_GetHotNColdCount(size_t* hotNColdCount);
/**
 * Get a defective pixel given the index.
 *
 * @param[in] index Index of defective pixel to retrive.
 * @param[out] defectivePixel Defective pixel retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_GetHotNColdByIndex(size_t index, DefectivePixel* defectivePixel);
/**
 * Dispose the instance of the DefectDetection.
 *
 * @param[in] hDefDet Handle to DefectDetection instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DD_Dispose(H_DEFECTDETECTION hDefDet);
/**
 * Creation function for DefectDetection instance. The image width,
 * height and pixel depth that will be used for the detection must be specified
 * and kept identical for all images. Keep in mind that when using color formats
 * (e.g. BayerRG8) the detection must be performed on every color separately.
 *
 * @param[in] width Width of the images the detection will be performed on.
 * @param[in] height Height of the images the detection will be performed on.
 * @param[in] depth Depth of the images the detection will be performed on.
 * @param[out] hDefDet Handle to DefectDetection instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError CreateDefectDetection(size_t width, size_t height, PixelDepth depth,
                                             H_DEFECTDETECTION* hDefDet);

#ifdef __cplusplus
}
}
#endif

#endif
