
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_DEVICE_H
#define ITALA_API_C_DEVICE_H

#include "Export.h"
#include "ItalaApiC/ItalaCDefs.h"

#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

typedef void (*EventCallback)(H_DEVICE hDevice, uint64_t eventId, void* pContext);
/**
 * Get the GenICam nodemap of the physical device. The nodemap instance is
 * accessible via GenApi NODEMAP_ functinos and allows the user to configure the behaviour
 * of the camera.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[out] phNodeMap Handle to NodeMap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetNodeMap(H_DEVICE hDevice, H_NODEMAP* phNodeMap);
/**
 * Get the GenICam nodemap of GenTL's Local Device module.
 * The nodemap instance is accessible via GenApi NODEMAP_ functinos and allows the user
 * to configure the behaviour of the module.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[out] phNodeMap Handle to NodeMap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetTLDeviceNodeMap(H_DEVICE hDevice, H_NODEMAP* phNodeMap);
/**
 * Get the GenICam nodemap of GenTL's Datastream module. The nodemap instance
 * is accessible via GenApi NODEMAP_ functinos and allows the user to configure the behaviour
 * of the module.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[out] phNodeMap Handle to NodeMap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetDatastreamNodeMap(H_DEVICE hDevice, H_NODEMAP* phNodeMap);
/**
 * Starts a continuous acquisition with a
 * default count of 16 image buffers (so a maximum of 16 images can
 * be grabbed and kept simultaneously without disposing them).
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_StartAcquisition(H_DEVICE hDevice);
/**
 * Starts the image acquisition on the device. A number of buffers equal to
 * bufferCount is allocated and kept in the acquisition engine input pool.
 * Internally, the acquisition engine picks buffers from the input pool
 * and fills them with new data, before pushing them to the output queue,
 * in accordance with the policy defined by the GenTL SFNC standard
 * feature "StreamBufferHandlingMode" (configurable through the Datastream
 * nodemap). The acquisition engine keeps running until numFrames frames are
 * pushed to the output queue (i.e. until the specified amount of frames is
 * available to the user).
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[in] bufferCount Number of buffers to be used for acquisition
 * @param[in] numFrames Number of frames to be acquired. If 0, the acquisition
 * runs continuously until DEV_StopAcquisition is called.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_StartAcquisitionEx(H_DEVICE hDevice, uint64_t bufferCount,
                                              uint64_t numFrames);
/**
 * The image acquisition is stopped and all the resources deallocated.
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_StopAcquisition(H_DEVICE hDevice);
/**
 * Grabs the first available image buffer in the output queue of the
 * acquisition engine. If the specified timeout expires, a TimeoutException
 * occurs. When an image buffer is grabbed, it's removed from the acquisition engine
 * and delivered to the client code so that its data can be used.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[in] timeout the amount of time in millisecond to wait for a new image
 * @param[out] phImage Handle to Image instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetNextImage(H_DEVICE hDevice, int64_t timeout, H_IMAGE* phImage);
/**
 * Starts the event functionality. The event listening thread is started and
 * ItalaApiC waits for incoming events from the device (which requires a
 * separate configuration via GenApi for event initialization).
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_EnableEvents(H_DEVICE hDevice);
/**
 * Stops the event functionality. The event listening thread is stopped.
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_DisableEvents(H_DEVICE hDevice);
/**
 * Subscribes an EventCallback instance to the current device.
 * The implementation must be provided by the client code.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[in] eventCallback Callback to execute.
 * @param[in] pContext Context of the callback.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_RegisterEventCallback(H_DEVICE hDevice, EventCallback eventCallback,
                                                 void* pContext);
/**
 * Removes the device event handler instance registered.
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_DeregisterEventCallback(H_DEVICE hDevice);
/**
 * Get the tick frequency of the device timestamp counter in ticks per
 * second. The counter is used, for instance, to assign timestamps to the
 * individual acquired images.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[out] pTimestampFrequency Timestamp frequency in tick per seconds.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetTimestampFrequency(H_DEVICE hDevice, uint64_t* pTimestampFrequency);
/**
 * Get the information of the device.
 *
 * @param[in] hDevice Handle to Device instance.
 * @param[out] pDeviceInfo Information struct containing all the info of the device.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_GetInfo(H_DEVICE hDevice, DeviceInfo* pDeviceInfo);
/**
 * Dispose the instance of the Device.
 *
 * @param[in] hDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError DEV_Dispose(H_DEVICE hDevice);

#ifdef __cplusplus
}
}
#endif

#endif
