
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_FLOAT_H
#define ITALA_API_C_FLOAT_H

#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#include "ItalaApiC/ItalaGenApiCDefs.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

/**
 * Set the value of the float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[in] value Value to set.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatSetValue(H_NODE hNode, double value);
/**
 * Get the value of the float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pValue Value of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetValue(H_NODE hNode, double* pValue);
/**
 * Get the minimum value of the float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pMin Minimum value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetMin(H_NODE hNode, double* pMin);
/**
 * Get the maximum value of the float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pMax Maximum value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetMax(H_NODE hNode, double* pMax);
/**
 * Get if the float Node has a software-imposed increment.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pHasInc Node has increment.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatHasInc(H_NODE hNode, bool* pHasInc);
/**
 * Get the increment mode of the float Node, if any.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pIncMode Increment mode.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetIncMode(H_NODE hNode, IncMode* pIncMode);
/**
 * Get the increment of the float Node, if any.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pIncrement Increment of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetInc(H_NODE hNode, double* pIncrement);
/**
 * Get the number of possible valid vlaues of the float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pNumOfValidValues Number of valid values.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatValidValuesNum(H_NODE hNode, size_t* pNumOfValidValues);
/**
 * Get valid value of the float Node, given a specific index.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[in] index Index of the valid value.
 * @param[out] pValidValue Valid value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetValidValueByIndex(H_NODE hNode, size_t index,
                                                      double* pValidValue);
/**
 * Get recommended representation of a flaot Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pRepresentation Recommended representation.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetRepresentation(H_NODE hNode, Representation* pRepresentation);
/**
 * Get the physical unit name of a float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] unit Array of char, unit name.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetUnit(H_NODE hNode, char* unit, size_t* pSize);
/**
 * Get the way the float should be converted to a string.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pDisplayNotation Display notation.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetDisplayNotation(H_NODE hNode,
                                                    DisplayNotation* pDisplayNotation);
/**
 * Get the precision to be used when converting the float to a string.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] pDisplayPrecision Display precision.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatGetDisplayPrecision(H_NODE hNode, int64_t* pDisplayPrecision);
/**
 * Restrict minimum value of float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] minImpose Value to imopse as minimum.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatImposeMin(H_NODE hNode, double minImpose);
/**
 * Restrict maximum value of float Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Float node.
 * @param[out] maxImpose Value to imopse as maximum.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_FloatImposeMax(H_NODE hNode, double maxImpose);

#ifdef __cplusplus
}
}
#endif

#endif