
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_IMAGE_H
#define ITALA_API_C_IMAGE_H

#include <stdbool.h>
#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#include "PFNC/PFNC.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

/**
 * Get the width of the image in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pWidth Width of the image in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetWidth(H_IMAGE hImage, size_t* pWidth);
/**
 * Get the height of the image in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pHeight Height of the image in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetHeight(H_IMAGE hImage, size_t* pHeight);
/**
 * Get the offsex along the X axis of the image, in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pOffsetX Offsex along the X axis of the image, in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetOffsetX(H_IMAGE hImage, size_t* pOffsetX);
/**
 * Get the offsex along the Y axis of the image, in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pOffsetY Offsex along the Y axis of the image, in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetOffsetY(H_IMAGE hImage, size_t* pOffsetY);
/**
 * Get the padding along the X axis of the image, in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pPaddingX Padding along the X axis of the image, in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetPaddingX(H_IMAGE hImage, size_t* pPaddingX);
/**
 * Get the padding along the Y axis of the image, in pixels.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pPaddingY Padding along the Y axis of the image, in pixels.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetPaddingY(H_IMAGE hImage, size_t* pPaddingY);
/**
 * Get the pixel format of the image. The PFNC standard from
 * GenICam is used to identify the format.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pFormat Pixel format of the image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetPixelFormat(H_IMAGE hImage, PfncFormat* pFormat);
/**
 * Get the ID of the image.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pFrameId ID of the image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetFrameID(H_IMAGE hImage, uint64_t* pFrameId);
/**
 * Get the timestamp of the image.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pTimestamp Timestamp of the image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetTimestamp(H_IMAGE hImage, uint64_t* pTimestamp);
/**
 * Get the payload size of the image, in bytes.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pPayloadSize Payload size of the image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetPayloadSize(H_IMAGE hImage, size_t* pPayloadSize);
/**
 * Reports wether the grabbed image is incomplete, i.e. some bytes
 * are missing due to transfer issues.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pIsIncomplete Reports image is incomplete.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_IsIncomplete(H_IMAGE hImage, bool* pIsIncomplete);
/**
 * Get the amount of bytes actually stored in the buffer.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pBytesFilled Amount of bytes actually stored in the buffer.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetBytesFilled(H_IMAGE hImage, size_t* pBytesFilled);
/**
 * Get the pointer to the memory buffer containing the grabbed data.
 * The content is retrieved directly from the device and, if chunk data
 * functionality is enabled, the chunks are also included in the buffer.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pBuffer Pointer to the memory buffer containing the grabbed data.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetData(H_IMAGE hImage, void** pBuffer);
/**
 * Report wether the buffer contains chunk data or not.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] pHasChunkData Buffer contains chunk data or not.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_HasChunkData(H_IMAGE hImage, bool* pHasChunkData);
/**
 * Get the specified node from the internal chunk data nodemap.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[in] name Name of the node to retrive.
 * @param[out] phNode Handle to Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_GetChunkNode(H_IMAGE hImage, const char* name, H_NODE* phNode);
/**
 * Creates a new image cloned from the current one with its
 * own allocated memory. It needs to be disposed when it's no
 * longer required via IMG_Dispose().
 *
 * @param[in] hImage Handle to Image instance.
 * @param[out] phCloneImage Handle to Image clone instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_Clone(H_IMAGE hImage, H_IMAGE* phCloneImage);
/**
 * Creates a new image converted from the current one with its
 * own allocated memory and the specified pixel format. It needs
 * to be disposed when it's no longer required via IMG_Dispose().
 *
 * @warning
 * When converting from packed to unpacked formats, the resulting
 * data is aligned according to the PFNC specification. Most of
 * the time the resulting data is LSB aligned, e.g. a 12-bit
 * pixel value stored in a 16-bit component has its MSBs padded
 * with zeroes.
 * See the official PFNC document hosted by EMVA for details.
 *
 * @param[in] hImage Handle to Image instance.
 * @param[in] dest_format Pixel format of new image instance.
 * @param[out] phCloneImage Handle to Image clone instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_Convert(H_IMAGE hImage, PfncFormat dest_format, H_IMAGE* phCloneImage);
/**
 * Dispose the instance of the Image.
 *
 * @param[in] hImage Handle to Image instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IMG_Dispose(H_IMAGE hImage);

/**
 * Creates a new image from a memory buffer by copying and taking ownership of the data. The size in
 * bytes of the buffer is obtained from the width, height and format provided by the caller. For
 * that reason, Y padding in the buffer is ignored while X padding is not supported.
 *
 * @note Chunk data access and grab related info are not available for created images.
 *
 * @param[in] data Pointer to the image data.
 * @param[in] width Width of the image in pixel.
 * @param[in] height Height of the image in pixel.
 * @param[in] pixelFormat Pixel format of the image in pixel.
 * @param[out] phImage handle to converted image instate.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError CreateImage(void* data, size_t width, size_t height, PfncFormat pixelFormat,
                                   H_IMAGE* phImage);

#ifdef __cplusplus
}
}
#endif

#endif
