
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_INTEGER_H
#define ITALA_API_C_INTEGER_H

#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#include "ItalaApiC/ItalaGenApiCDefs.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

/**
 * Set the value of the integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[in] value Value to set.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerSetValue(H_NODE hNode, int64_t value);
/**
 * Get the value of the integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pValue Value of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetValue(H_NODE hNode, int64_t* pValue);
/**
 * Get the minimum value of the integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pMin Minimum value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetMin(H_NODE hNode, int64_t* pMin);
/**
 * Get the maximum value of the integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pMax Maximum value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetMax(H_NODE hNode, int64_t* pMax);
/**
 * Get the increment mode of the integer Node, if any.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pIncMode Increment mode.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetIncMode(H_NODE hNode, IncMode* pIncMode);
/**
 * Get the increment of the integer Node, if any.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pIncrement Increment of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetInc(H_NODE hNode, int64_t* pIncrement);
/**
 * Get the number of possible valid vlaues of the integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pNumOfValidValues Number of valid values.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerValidValuesNum(H_NODE hNode, size_t* pNumOfValidValues);
/**
 * Get valid value of the integer Node, given a specific index.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[in] index Index of the valid value.
 * @param[out] pValidValue Valid value.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetValidValueByIndex(H_NODE hNode, size_t index,
                                                        int64_t* pValidValue);
/**
 * Get recommended representation of a integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] pRepresentation Recommended representation.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetRepresentation(H_NODE hNode, Representation* pRepresentation);
/**
 * Get the physical unit name of a integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] unit Array of char, unit name.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerGetUnit(H_NODE hNode, char* unit, size_t* pSize);
/**
 * Restrict minimum value of integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] minImpose Value to imopse as minimum.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerImposeMin(H_NODE hNode, int64_t minImpose);
/**
 * Restrict maximum value of integer Node.
 *
 * @param[in] hNode Handle to Node instance, should be a Integer node.
 * @param[out] maxImpose Value to imopse as maximum.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IntegerImposeMax(H_NODE hNode, int64_t maxImpose);

#ifdef __cplusplus
}
}
#endif

#endif