
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_ITALACDEFS_H
#define ITALA_API_C_ITALACDEFS_H

#include <stdint.h>

#define MAX_SUPPORTED_DEVICES 255
#define MAX_SUPPORTED_INTERFACES 255
#define MAX_STRING_LENGTH 1024

#define DEVINFO_ID_LENGTH MAX_STRING_LENGTH
#define DEVINFO_VENDOR_LENGTH MAX_STRING_LENGTH
#define DEVINFO_MODEL_LENGTH MAX_STRING_LENGTH
#define DEVINFO_DISPLAYNAME_LENGTH MAX_STRING_LENGTH
#define DEVINFO_USERDEFINEDNAME_LENGTH MAX_STRING_LENGTH
#define DEVINFO_SERIALNUMBER_LENGTH MAX_STRING_LENGTH
#define DEVINFO_VERSION_LENGTH MAX_STRING_LENGTH

#define ITFINFO_ID_LENGTH MAX_STRING_LENGTH
#define ITFINFO_DISPLAYNAME_LENGTH MAX_STRING_LENGTH

#define DD_STRIG_LENGTH 150

/* To ensure compile-time type safety. Error if passing the wrong handler as parameter.*/
#define DEFINE_HANDLE(name) \
  struct name##__ {         \
    uint8_t unused;         \
  };                        \
  typedef struct name##__ *name

#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

DEFINE_HANDLE(H_DEVICE);           ///< Handle to instance of Device
DEFINE_HANDLE(H_IMAGE);            ///< Handle to instance of Image
DEFINE_HANDLE(H_DEFECTDETECTION);  ///< Handle to instance of DefectDetection

DEFINE_HANDLE(H_NODEMAP);       ///< Handle to instance of Nodemap
DEFINE_HANDLE(H_NODE);          ///< Handle to instance of Node
DEFINE_HANDLE(H_NODECALLBACK);  ///< Handle to instance of NodeCallback

/**
 * Enumeration for Itala Errors value.
 */
enum _ItalaError {
  ItalaErrorSuccess = 0,          //!< No error occures
  ItalaErrorError = 1,            //!< Generic error
  ItalaErrorBadAlloc = 2,         //!< Bad allocation memory
  ItalaErrorInvalidArgument = 3,  //!< Invalid argument passed to function
  ItalaErrorOutOfRange = 4,       //!< Out of range
  ItalaErrorProperty = 5,         //!< Property error
  ItalaErrorRuntime = 6,          //!< Runtime execution error
  ItalaErrorLogical = 7,          //!< Logical runtime error
  ItalaErrorAccess = 8,           //!< Access error, invalid permission
  ItalaErrorTimeout = 9,          //!< Timeout
};
/**
 * \typedef ItalaError
 * Type of error returned from ItalaApiC.
 * @see _ItalaError
 */
typedef int32_t ItalaError;
/**
 * Enumeration for Itala device access status value.
 */
enum _DeviceAccessStatus {
  UnknownAccessStatus = 0,   //!< Unknown access status
  AvailableReadWrite = 1,    //!< Available read and write
  AvailableReadOnly = 2,     //!< Available read only
  Unreachable = 3,           //!< Unreachable
  Busy = 4,                  //!< Busy
  AlreadyOpenReadWrite = 5,  //!< Already open read and write
  AlreadyOpenReadOnly = 6    //!< Already open read only
};
/**
 * \typedef DeviceAccessStatus
 * @see _DeviceAccessStatus
 */
typedef int32_t DeviceAccessStatus;
/**
 * Enumeration for Itala defect type value.
 */
enum _DefectType {
  Leaky = 0,  //!< Leaky
  Hot = 1,    //!< Hot
  Cold = 2    //!< Cold
};
/**
 * \typedef DefectType
 * @see _DefectType
 */
typedef int32_t DefectType;
/**
 * Enumeration for Itala pixel depth value.
 */
enum _PixelDepth {
  D8Bit = 0,   //!< D8Bit
  D10Bit = 1,  //!< D10Bit
  D12Bit = 2   //!< D12Bit
};
/**
 * \typedef PixelDepth
 * @see _PixelDepth
 */
typedef int32_t PixelDepth;

/** \struct InterfaceInfo ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *   \brief This is a struct to contain all the data of inteferfaces.
 * This struct contains all the data of the interfaces.
 */
typedef struct {
  char ID[ITFINFO_ID_LENGTH];                    ///< The unique ID of the interface.
  char DisplayName[ITFINFO_DISPLAYNAME_LENGTH];  ///< String containing the user readable name of
                                                 ///< the interface.
  uint32_t IpAddress;   ///< IP address of the interface in host byte order.
  uint64_t MacAddress;  ///< Mac address of the interface in host byte order.
  uint32_t SubnetMask;  ///< Subnet mask of the interface in the same format described in IpAddress.
} InterfaceInfo;

/** \struct DeviceInfo ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *   \brief This is a struct to contain all the data of devices.
 * This struct contains all the data of the devices.
 */
typedef struct {
  char ID[DEVINFO_ID_LENGTH];                            ///< Unique ID of the device.
  char Vendor[DEVINFO_VENDOR_LENGTH];                    ///< Name of the device vendor.
  char Model[DEVINFO_MODEL_LENGTH];                      ///< Model of the device.
  char DisplayName[DEVINFO_DISPLAYNAME_LENGTH];          ///< User readable name of the device
  char UserDefinedName[DEVINFO_USERDEFINEDNAME_LENGTH];  ///< Name of the device defined by the
                                                         ///< user, when available.
  char SerialNumber[DEVINFO_SERIALNUMBER_LENGTH];        ///< Serial number of the device.
  char Version[DEVINFO_VERSION_LENGTH];                  ///< Version of the physical device.
  DeviceAccessStatus AccessStatus;                       ///< Accessibility status of the device.
  uint32_t IpAddress;   ///< IP address of the device in host order.
  uint32_t SubnetMask;  ///< Subnet mask configured on the device in the same format described in
                        ///< IpAddress.
  uint32_t DefaultGateway;  ///< Default gateway configured on the device according with the same
                            ///< format described in IpAddress()
  uint64_t MacAddress;      ///< Mac address of the device in host byte order.
  InterfaceInfo InterfaceInfo;  ///< InterfaceInfo instance of the interface under which the device
                                ///< is connected and enumerated.
} DeviceInfo;

/** \struct DefectivePixel ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *   \brief This is a struct to contain all the data of defective pixels.
 * This struct contains all the data of the defective pixel elaboration, is used to
 * store the pixels with different types of problems retrived from the sensor analysis.
 */
typedef struct {
  uint64_t X;                        ///< The X image coordinate of the defective pixel
  uint64_t Y;                        ///< The Y image coordinate of the defective pixel.
  DefectType DefectType;             ///< The type of defect the pixel has.
  char TypeString[DD_STRIG_LENGTH];  ///< The string representation of the type of defect the pixel
                                     ///< has.
} DefectivePixel;

/**
 * Components of the polarizer filter which represent a specific angle of polarization of the
 * incoming light.
 */
enum _PolarComponent {
  P0 = 0,   //!< P0
  P45 = 1,  //!< P45
  P90 = 2,  //!< P90
  P135 = 3  //!< P135
};
/**
 * \typedef PolarComponent
 * @see _PolarComponent
 */
typedef int32_t PolarComponent;

/**
 * Demosaicing algorithm applied to the image. None, NearestNeighbour and Bilinear
 * algorithms give a resulting image with the same pixel format of the original. WeightedBilinear
 * returns 32-bit floating point data. The width and height of the resulting image also depend on
 * the used algorithm. In particular:
 * - None -> w/2, h/2
 * - NearestNeighbour -> w - 2, h - 2
 * - Bilinear -> w - 3, h - 3
 * - WeightedBilinear -> w - 4, h - 4
 */
enum _PolarDemosaicingAlgorithm {
  NonePolar = 0,         //!< NonePolar
  NearestNeighbour = 1,  //!< NearestNeighbour
  Bilinear = 2,          //!< Bilinear
  WeightedBilinear = 3,  //!< WeightedBilinear
};
/**
 * \typedef PolarDemosaicingAlgorithm
 * @see _PolarDemosaicingAlgorithm
 */
typedef int32_t PolarDemosaicingAlgorithm;

/** \struct PolarComponents ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *  \brief Convenient POD structure used as container for @ref ExtractAllPolarComponents 's
 * multi-result.
 */
typedef struct {
  H_IMAGE P0;    ///< P0 image
  H_IMAGE P45;   ///< P45 image
  H_IMAGE P90;   ///< P90 image
  H_IMAGE P135;  ///< P135 image
} PolarComponents;

/** \struct PolarComponentsColor ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *  \brief Convenient POD structure used as container for @ref ExtractAllPolarComponentsColor 's
 *  multi-result.
 */
typedef struct {
  PolarComponents RedComponents;    ///< Red component containing all the 4 polarized image
  PolarComponents GreenComponents;  ///< Green component containing all the 4 polarized image
  PolarComponents BlueComponents;   ///< Blue component containing all the 4 polarized image
} PolarComponentsColor;

/** \struct StokesVectors ItalaCDefs.h "ItalaApiC/ItalaCDefs.h"
 *  \brief Convenient POD structure used as container for @ref ComputeAllStokes 's
 * multi-result.
 */
typedef struct {
  H_IMAGE S0;  ///< S0 stokes vector
  H_IMAGE S1;  ///< S1 stokes vector
  H_IMAGE S2;  ///< S2 stokes vector
} StokesVectors;

#ifdef __cplusplus
}
}
#endif

#endif