
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_NODE_H
#define ITALA_API_C_NODE_H

#include <stdbool.h>
#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#include "ItalaApiC/ItalaGenApiCDefs.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

typedef void (*NodeCallbackFunction)(H_NODE hNode, void* pContext);

/**
 * Get the node name.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] name The name of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetName(H_NODE hNode, char* name, size_t* pSize);
/**
 * Get the node namespace.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pNameSpace Namespace retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetNameSpace(H_NODE hNode, NameSpace* pNameSpace);
/**
 * Get the node visibility.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pVisibility Visibility retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetVisibility(H_NODE hNode, Visibility* pVisibility);
/**
 * Invalidate the current node.
 *
 * @param[in] hNode Handle to Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_InvalidateNode(H_NODE hNode);
/**
 * Get if the node is cachable or not.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsCachable True if cachable, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsCachable(H_NODE hNode, bool* pIsCachable);
/**
 * Get if the node access mode is cachable or not.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsAccessModeCacheable True if access mode is cachable, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsAccessModeCacheable(H_NODE hNode, bool* pIsAccessModeCacheable);
/**
 * Get the node caching mode.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pCachingMode Caching mode retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetCachingMode(H_NODE hNode, CachingMode* pCachingMode);
/**
 * Get the node polling time.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pPollingTime Polling time retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetPollingTime(H_NODE hNode, int64_t* pPollingTime);
/**
 * Get the node tooltip.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] toolTip The tooltip of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetToolTip(H_NODE hNode, char* toolTip, size_t* pSize);
/**
 * Get the node description.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] description The description of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetDescription(H_NODE hNode, char* description, size_t* pSize);
/**
 * Get the node diplay name.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] displayName The display name of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetDisplayName(H_NODE hNode, char* displayName, size_t* pSize);
/**
 * Get the node device name.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] deviceName The device name of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetDeviceName(H_NODE hNode, char* deviceName, size_t* pSize);
/**
 * Get the number of childer Node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] linkType The link type.
 * @param[out] pNumChildren Number of children nodes.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetChildrenNum(H_NODE hNode, LinkType linkType, size_t* pNumChildren);
/**
 * Get a children node, given the specific index.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] index Index of Node.
 * @param[in] linkType The link type.
 * @param[out] phNodeChildren Handle to children Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetChildrenByIndex(H_NODE hNode, size_t index, LinkType linkType,
                                               H_NODE* phNodeChildren);
/**
 * Get the number of parent Node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pNumParent Number of parent nodes.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetParentNum(H_NODE hNode, size_t* pNumParent);
/**
 * Get a parent node, given the specific index.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] index Index of Node.
 * @param[out] phNodeParent Handle to parent Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetParentByIndex(H_NODE hNode, size_t index, H_NODE* phNodeParent);
/**
 * Subscribes a NodeCallbackFunction instance to the current node.
 * The implementation must be provided by the client code.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] callback The method to execute as callback.
 * @param[in] pContext Context of the callback.
 * @param[out] phNodeCallback Handle to NodeCallback instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_RegisterCallback(H_NODE hNode, NodeCallbackFunction callback,
                                             void* pContext, H_NODECALLBACK* phNodeCallback);
/**
 * Unsubscribe a NodeCallbackFunction instance to the current node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] hNodeCallback Handle to NodeCallback instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_DeregisterCallback(H_NODE hNode, H_NODECALLBACK hNodeCallback);
/**
 * Get the nodemap relative to this node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] phNodeMap Handle to NodeMap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetNodeMap(H_NODE hNode, H_NODEMAP* phNodeMap);
/**
 * Get the node event id.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] eventId The event id of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetEventId(H_NODE hNode, char* eventId, size_t* pSize);
/**
 * Get if the node is streamable or not.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsStreamable True if is streamable, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsStreamable(H_NODE hNode, bool* pIsStreamable);
/**
 * Get the property name number of the Node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pNumOfProperty Name number of the node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetPropertyNamesNum(H_NODE hNode, size_t* pNumOfProperty);
/**
 * Get a property name, given the specific index.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] index Index of Node.
 * @param[out] propertyName Handle to parent Node instance.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetPropertyNameByIndex(H_NODE hNode, size_t index, char* propertyName,
                                                   size_t* pSize);
/**
 * Get a property plus an additional attribute by name
 * If a property has multiple values/attribute they come with Tabs as delimiters.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[in] propertyName Name of the property.
 * @param[out] values Values of the property.
 * @param[in, out] pSizeValue Maximum element of the values array in input, number of element filled
 * in output.
 * @param[out] attributes Attributes of the property.
 * @param[in, out] pSizeAttributes Maximum element of the attributes array in input, number of
 * element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetProperty(H_NODE hNode, const char* propertyName, char* values,
                                        size_t* pSizeValue, char* attributes,
                                        size_t* pSizeAttributes);
/**
 * Imposes an access mode to the natural access mode of the node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] imposedAccessMode Imposed access mode to set.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_ImposedAccessMode(H_NODE hNode, AccessMode imposedAccessMode);
/**
 * Imposes a visibility to the natural visibility of the node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] imposedVisibility Imposed visibility to set.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_ImposedVisibility(H_NODE hNode, Visibility imposedVisibility);
/**
 * Retrieves the node which describes the same feature in a different way.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] phNodeAlias Handle to alias Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetAlias(H_NODE hNode, H_NODE* phNodeAlias);
/**
 * Retrieves the a node which describes the same feature so that it can be casted
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] phNodeCastAlias Handle to cast alias Node instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetCastAlias(H_NODE hNode, H_NODE* phNodeCastAlias);
/**
 * Get the node documentation URL.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] docuURL The documentation URL of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetDocuURL(H_NODE hNode, char* docuURL, size_t* pSize);
/**
 * Get if the node is deprecated or not.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsDeprecated True if is deprecated, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsDeprecated(H_NODE hNode, bool* pIsDeprecated);
/**
 * Get the type of the main interface of a node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pInterfaceType Interface type of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetPrincipalInterfaceType(H_NODE hNode, InterfaceType* pInterfaceType);
/**
 * Get if the node can be reached via category nodes from a category node named "Root".
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsFeature True if can be reached via category nodes, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsFeature(H_NODE hNode, bool* pIsFeature);

// FROM IBASE
/**
 * Get the access mode of a node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pAccessMode Access mode of the Node.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_GetAccessMode(H_NODE hNode, AccessMode* pAccessMode);

// FROM IVALUE
ITALA_API_C ItalaError NODE_FromString(H_NODE hNode, const char* valueStr);
/**
 * Get the node as a string.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] value The string of the Node.
 * @param[in, out] pSize Maximum element of the array in input, number of element filled in output.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_ToString(H_NODE hNode, char* value, size_t* pSize);
/**
 * Checks if the value comes from cache or is requested from another node.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsCacheValid True if the value comes from cache, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError NODE_IsValueCacheValid(H_NODE hNode, bool* pIsCacheValid);

// FREE FUN IN INODE
/**
 * Checks if the node is readable.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsReadable True if the node is readable, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IsNodeReadable(H_NODE hNode, bool* pIsReadable);
/**
 * Checks if the node is writable.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsWritable True if the node is writable, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IsNodeWritable(H_NODE hNode, bool* pIsWritable);
/**
 * Checks if the node is implemented.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsImplemented True if the node is implemented, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IsNodeImplemented(H_NODE hNode, bool* pIsImplemented);
/**
 * Checks if the node is available.
 *
 * @param[in] hNode Handle to Node instance.
 * @param[out] pIsAvailable True if the node is available, false otherwise.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError IsNodeAvailable(H_NODE hNode, bool* pIsAvailable);

#ifdef __cplusplus
}
}
#endif

#endif