
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2024 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_POLARITAZION_H
#define ITALA_API_C_POLARITAZION_H

#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#include "PFNC/PFNC.h"

#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

/**
 * Extract a specific component (or angle) of the polarizer filter. The resulting image
 * represents the captured light at the angle specified in degrees.
 *
 * @note Only monochrome images are supported.
 *
 * @param[in] hImage Handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[in] component Component of the polarizer sensor.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] hComponent handle to Image containing the requested polar component.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ExtractPolarComponent(H_IMAGE hImage, PolarComponent component,
                                             PolarDemosaicingAlgorithm algorithm,
                                             H_IMAGE* hComponent);

/**
 * Extract an image for each component (or angle) of the polarizer filter. Each resulting
 * image represents the captured light at 0, 45, 90 and 135 degrees respectively.
 *
 * @param[in] hImage Handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] componentsMono containing the pointers to the polar component images.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ExtractAllPolarComponents(H_IMAGE hImage,
                                                 PolarDemosaicingAlgorithm algorithm,
                                                 PolarComponents* componentsMono);

/**
 * For each color channel, extract an image for each component (or angle) of the polarizer
 * filter. Each resulting image represents the captured light at 0, 45, 90 and
 * 135 degrees respectively, for each color channel.
 *
 * @param[in] hImage Handle to Image instance, acquired using a color polarization filter array.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] componentsColor containing the pointers to the polar component images, for each color
 * channel.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ExtractAllPolarComponentsColor(H_IMAGE hImage,
                                                      PolarDemosaicingAlgorithm algorithm,
                                                      PolarComponentsColor* componentsColor);

/**
 * Computes only the S0 component of the Stokes vectors from the extracted polar components.
 *
 * @param[in] polarComponents Polarization components demosaiced from the raw image.
 * @param[out] hS0 handle to Image ccontaining the requested stokes vectors.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeStokesS0(PolarComponents* polarComponents, H_IMAGE* hS0);

/**
 * Computes only the S1 component of the Stokes vectors from the extracted polar components.
 *
 * @param[in] polarComponents Polarization components demosaiced from the raw image.
 * @param[out] hS1 handle to Image ccontaining the requested stokes vectors.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeStokesS1(PolarComponents* polarComponents, H_IMAGE* hS1);

/**
 * Computes only the S2 component of the Stokes vectors from the extracted polar components.
 *
 * @param[in] polarComponents Polarization components demosaiced from the raw image.
 * @param[out] hS2 handle to Image containing the requested stokes vectors.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeStokesS2(PolarComponents* polarComponents, H_IMAGE* hS2);

/**
 * Compute all the Stokes vectors to determine the polarization state of the light given the
 * extracted components. The Stokes vector components are expressed as 32-bit or 64-bit floating
 * point numbers depending on the raw image format.
 *
 * @param[in] polarComponents polarization components demosaiced from the raw image.
 * @param[out] stokesVectors struct containing all handles of the stokes vectors.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeAllStokes(PolarComponents* polarComponents,
                                        StokesVectors* stokesVectors);

/**
 * Compute the AoLP (Angle of Linear Polarization) image from the Stokes vectors.
 *
 * @param[in] stokesVectors Stokes vectors computed from the polarization components.
 * @param[out] hImageAoLP handle to the AoLP image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeAoLP(StokesVectors* stokesVectors, H_IMAGE* hImageAoLP);

/**
 * Compute the DoLP (Degree of Linear Polarization) image from the Stokes vectors.
 *
 * @param[in] stokesVectors Stokes vectors computed from the polarization components.
 * @param[out] hImageDoLP handle to the DoLP image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeDoLP(StokesVectors* stokesVectors, H_IMAGE* hImageDoLP);

/**
 * Compute the intensity image from the Stokes vectors.
 *
 * @param[in] stokesVectors Stokes vectors computed from the polarization components.
 * @param[in] rawImagePixelFormat The format of the raw polarization image.
 * @param[out] hIntensity handle to intensity image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputeIntensity(StokesVectors* stokesVectors,
                                        PfncFormat rawImagePixelFormat, H_IMAGE* hIntensity);

/**
 * Convenience function which computes the AoLP (Angle of Linear Polarization) image directly
 * from the raw polarization image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param[in] hImage handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] hImageAoLP  handle to the AoLP image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError InstantComputeAoLP(H_IMAGE hImage, PolarDemosaicingAlgorithm algorithm,
                                          H_IMAGE* hImageAoLP);

/**
 * Convenience function which computes the DoLP (Degree of Linear Polarization) image directly
 * from the raw polarization image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param[in] hImage handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] hImageDoLP handle to the AoLP image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError InstantComputeDoLP(H_IMAGE hImage, PolarDemosaicingAlgorithm algorithm,
                                          H_IMAGE* hImageDoLP);

/**
 * Convenience function which computes the intensity image directly from the raw polarization
 * image without manual intermediate steps.
 *
 * @note Only monochrome images are supported.
 *
 * @param[in] hImage handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[in] algorithm Demosaicing algorithm to be applied to the image.
 * @param[out] hIntensity handle to the to intensity image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError InstantComputeIntensity(H_IMAGE hImage, PolarDemosaicingAlgorithm algorithm,
                                               H_IMAGE* hIntensity);

/**
 * Computes an image split in quadrants containing the four extracted polarization
 * components (angles), of the same width and height of the raw polarization image.
 *
 * @note Only monochrome images are supported.
 *
 * @param[in] hImage handle to Image instance, acquired using a monochrome polarization filter
 * array.
 * @param[out] hPolarQuadrants handle to quadrants image.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError ComputePolarQuadrantsImage(H_IMAGE hImage, H_IMAGE* hPolarQuadrants);

#ifdef __cplusplus
}
}
#endif

#endif