
/***********************************************************************************
 *
 * ItalaApi - Copyright (C) 2023 Opto Engineering
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY SUFFERED BY LICENSE AS
 * A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 ***********************************************************************************/

#ifndef ITALA_API_C_SYSTEM_H
#define ITALA_API_C_SYSTEM_H

#include <stddef.h>
#include "ItalaApiC/Export.h"
#include "ItalaApiC/ItalaCDefs.h"
#ifdef __cplusplus
extern "C" {
namespace ItalaC {
#endif

ITALA_API_C void ERR_GetLastErrorMessage(char* message, size_t* pSize);
ITALA_API_C ItalaError ERR_GetLastErrorCode();

/**
 * Entry point of the runtime. It allocates all necessary resources and creates
 * the system instance. Only one system can be created at a time. If a system
 * instance is already alive, a call to this function leads to an error.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_Initialize();
/**
 * Dispose the instance of the System.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_Dispose();
/**
 * Enumerates devices across all interfaces. The maximum number of enumerated
 * devices is 255. To retrieve the number of devices discovered and retrive them by index use:
 * @see SYS_GetSizeLastEnumeration and @see SYS_GetByIndexLastEnumeration.
 *
 * @param[in] timeout Maximum amount of time in ms for the enumeration to terminate.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_EnumerateDevices(uint64_t timeout);
/**
 * Enumerates all devices under a specific network interface. The maximum number
 * of enumerated devices is 255.To retrieve the number of devices discovered and
 * retrive them by index use: @see SYS_GetSizeLastEnumeration and @see
 * SYS_GetByIndexLastEnumeration.
 *
 * @param[in] itfInfo Interface info of the specific network interface.
 * @param[in] timeout Maximum amount of time in ms for the enumeration to terminate.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_EnumerateDevicesByInterface(InterfaceInfo itfInfo, uint64_t timeout);
/**
 * Enumerates all devices under a list of specific interfaces. The maximum number
 * of enumerated devices is 255..To retrieve the number of devices discovered and retrive
 * them by index use: @see SYS_GetSizeLastEnumeration and @see SYS_GetByIndexLastEnumeration.
 *
 * @param[in] itfInfoList Interface info array of the specific network interface.
 * @param[in] size Amount of interfaces contained in itfInfoList.
 * @param[in] timeout Maximum amount of time in ms for the enumeration to terminate.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_EnumerateDevicesByInterfaces(InterfaceInfo* itfInfoList, size_t size,
                                                        uint64_t timeout);
/**
 * Retrieve the number of devices enumerated the last time an EnumerationDevices method have been
 * called.
 *
 * @param[out] deviceCount The amount of devices the last enumeration have discovered.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetDeviceCount(size_t* deviceCount);
/**
 * Retrieve the device info by index.
 *
 * @param[in] index Index of the device to retrieve.
 * @param[out] deviceInfo Device info retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetDeviceByIndex(size_t index, DeviceInfo* deviceInfo);
/**
 * Creates a device instance which represents a physical device, given its
 * identyfing info. The maximum number of simultaneously created devices is 50.
 * When no longer needed, device instances must be disposed
 * via DEV_Dispose().
 *
 * @param[in] deviceInfo Device info of the specific device to create.
 * @param[out] phDevice Handle to Device instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_CreateDevice(DeviceInfo deviceInfo, H_DEVICE* phDevice);
/**
 * Get the GenICam nodemap of the GenTL System module. The nodemap instance
 * is accessible via GenApi NODEMAP_ functinos and allows the user to configure the
 * behaviour of the system.
 *
 * @param[out] phNodemap Handle to NodeMap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetNodeMap(H_NODEMAP* phNodemap);
/**
 * Enumerates the available network interfaces.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_EnumerateInterfaces();
/**
 * Retrieve the number of interfaces enumerated.
 *
 * @param[out] interfaceCount The amount of interfaces the last enumeration have discovered.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetInterfaceCount(size_t* interfaceCount);
/**
 * Retrieve interface info by index.
 *
 * @param[in] index Index of the device to retrieve.
 * @param[out] itfInfo Interface info retrived.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetInterfaceByIndex(size_t index, InterfaceInfo* itfInfo);
/**
 * Get the GenICam nodemap of a network interface. The nodemap instance is
 * accessible via GenApi NODEMAP_ functinos and allows the user to configure the behaviour
 * of the interface.
 *
 * @param[in] itfInfo nterface info of the specific network interface.
 * @param[out] phNodeMap MHandle to Nodemap instance.
 *
 * @return ItalaError the error code; return ItalaErrorSuccess (0) if no errors occures.
 */
ITALA_API_C ItalaError SYS_GetInterfaceNodemap(InterfaceInfo itfInfo, H_NODEMAP* phNodeMap);

#ifdef __cplusplus
}
}
#endif

#endif
