#!/bin/bash

# Get the current kernel version
KERNEL_VERSION=$(uname -r) 
GEV_DRIVER_VERSION="24.04.0"

MODULE_NAME="oegevmodule-${GEV_DRIVER_VERSION}-${KERNEL_VERSION}"
MODULE_TAR_NAME="${MODULE_NAME}.tar.gz"
MODULE_KO_NAME="${MODULE_NAME}.ko"
COMPLETE_OPTO_URL="https://www.opto-e.com/repository/_shared/itala-drivers/GEV/${MODULE_TAR_NAME}"
OPTO_SUPPORT_URL="https://www.opto-e.com/en/support"
WORKING_DIR=$(pwd)

# Temporary directory for downloading and extracting
TEMP_DIR="tmp"
mkdir -p $TEMP_DIR

# Download the .tar.gz file
echo "Downloading $COMPLETE_OPTO_URL..."
curl -L -o "${TEMP_DIR}/${MODULE_TAR_NAME}" "$COMPLETE_OPTO_URL"
    
if [ $? -ne 0 ]; then
    echo -e "Failed to download ${MODULE_TAR_NAME}. Check your internet connection or \n\
    You can try to download it manually at $COMPLETE_OPTO_URL\n\
    If your kernel version is not available on the website,\n\
    you can ask it to Opto Engineering support $OPTO_SUPPORT_URL"
    exit 1
fi

# Extract the .tar.gz file
echo "Extracting $TEMP_DIR/${MODULE_TAR_NAME}..."
tar -xzvf "$TEMP_DIR/${MODULE_TAR_NAME}" -C "$TEMP_DIR" 
if [ $? -ne 0 ]; then
    echo "Failed to extract the file."
    exit 1
fi


# Check again the file
KO_FILE=$(find "$TEMP_DIR" -name "${MODULE_KO_NAME}" | head -n 1)
if [ -z "$KO_FILE" ]; then
    echo "No kernel module (${MODULE_KO_NAME}) file found."
    exit 1
fi


# Install the kernel module
echo "Installing kernel module: ${MODULE_KO_NAME}..."

# save the .ko module in the system dir
mkdir -p /lib/modules/$(uname -r)/extra
cp -T "$TEMP_DIR/${MODULE_KO_NAME}" "/lib/modules/$(uname -r)/extra/oegevmodule.ko"
if [ $? -ne 0 ]; then
    echo "Failed to save .ko file in the system path. Do you have permission?"
    exit 1
fi
echo "Saved ${MODULE_KO_NAME} file in /lib/modules/$(uname -r)/extra/oegevmodule.ko"

# .conf file to run when loading the module
echo "Configuring /etc/modprobe.d/oegevmodule.conf..."
echo -e "\ninstall oegevmodule /sbin/modprobe --ignore-install oegevmodule; \\" > /etc/modprobe.d/oegevmodule.conf
for i in $(seq 0 9); do
    echo "test -e /dev/oegevmodule$i || mknod -m 666 /dev/oegevmodule$i c 120 $i; \\" >> /etc/modprobe.d/oegevmodule.conf
done
echo "" >> /etc/modprobe.d/oegevmodule.conf

# load module at boot
echo "Configuring /etc/modules-load.d/oegevmodule.conf..."
echo oegevmodule > /etc/modules-load.d/oegevmodule.conf

# reload all modules dependencies
echo "Loading all modules dependencies with depmod..."
depmod -a

# activate module
echo "Loading oegevmodule with modprobe..."
modprobe oegevmodule
if [ $? -ne 0 ]; then
    echo "Failed to install the kernel module."
    exit 1
fi

echo "Cleaning up files..."
rm -rf $TEMP_DIR

echo "Kernel module installed successfully."
